# coding: utf-8
# create by tongshiwei on 2017/12/2

import sys

if sys.version_info[0] == 3:
    string_types = str,
    integer_types = int
    # this function is needed for python3
    # to convert ctypes.char_p .value back to python str
    unistr = lambda x: x
    tostr = lambda x: str(x)
else:
    string_types = basestring,
    integer_types = (int, long)
    unistr = lambda x: x.decode('utf-8')
    tostr = lambda x: unicode(x)


# coding:utf-8
'''
此文件用以处理文件正则，需要指定一个正则规则文件

文件正则文件各式如下

不可命中规则1 不可命中规则2: 命中规则1 命中规则2

规则之间由空格分割，不可命中可以有也可以没有，不可命中和可命中规则均可有多个
在 分号: 前的规则有一个被命中视为与该正则规则不符， 分号: 后的规则有一个被命中，视为符合

规则实例文件见example/process_pattern_regex.exp
'''
from __future__ import print_function
from __future__ import unicode_literals

import re

from source.base import *


class cond_filter():
    # 过滤器，过滤 只含表情，数字 以及 回复（含有 //@ ）评论
    def __init__(self):
        self.emoji_re = re.compile('['
                                   '\U0001F300-\U0001F64F'
                                   '\U0001F680-\U0001F6FF'
                                   '\u2600-\u2B55]+',
                                   re.UNICODE)

    def is_valid(self, data):
        if re.sub("^\d+$", "", self.emoji_re.sub("", data['text'])):
            if "//@" not in data['text']:
                return True
        return False


def _init_patterns(line, pps=None):
    try:
        line = line.decode('utf-8').strip()
    except:
        pass
    try:
        line1, line2 = line.split(':')
    except:
        print(line)
        raise Exception('error in %s' % line)

    v = line1.strip()
    ps1 = []
    if v:
        try:
            p = re.compile(v)
        except:
            print("error:", v)
            raise Exception('error in %s' % line)
        ps1.append(p)

    v = line2.strip()
    assert v
    ps2 = []
    try:
        p = re.compile(v)
    except:
        print("error:", v)
        raise Exception('error in %s' % line)
    ps2.append(p)

    if pps is not None and isinstance(pps, dict):
        pps[line] = (ps1, ps2)
    return line, (ps1, ps2)


def init_patterns(location):
    pps = {}
    with open(location) as f:
        for line in f:
            try:
                _init_patterns(line, pps)
            except:
                continue
    return pps


def _init_patterns2(line, pps):
    try:
        line = line.decode('utf-8').strip()
    except:
        pass
    try:
        line1, line2 = line.split(':')
    except:
        print(line)
        raise Exception('error in %s' % line)

    vs = line1.split()
    ps1 = []
    for v in vs:
        try:
            p = re.compile(v)
        except:
            print("error:", v)
            raise Exception('error in %s' % line)
        ps1.append(p)

    vs = line2.split()
    ps2 = []
    for v in vs:
        try:
            p = re.compile(v)
        except:
            print("error:", v)
            raise Exception('error in %s' % line)
        ps2.append(p)

    if pps is not None and isinstance(pps, dict):
        pps[line] = (ps1, ps2)

    return line, (ps1, ps2)


def init_patterns2(location):
    '''
    创建一个模式字典
    :param location: str, 正则规则文件位置
    :return: dict, 模式字典
    '''
    pps = {}
    with open(location) as f:
        for line in f:
            try:
                _init_patterns2(line, pps)
            except:
                continue
    return pps


def regex(ps, line):
    if not ps:
        return False
    xs = []
    for p in ps:
        x = p.search(line)
        xs.append(x)
    for x in xs:
        if x == None:
            return False
    return True


class PatternHiter(object):
    '''
    模式识别器
    '''

    def __init__(self, location, mode=1):
        if mode == 1:
            if ':' in location:
                self.pps = dict()
                _init_patterns(location, self.pps)
            else:
                self.pps = init_patterns(location)
        else:
            if ':' in location:
                self.pps = dict()
                _init_patterns2(location, self.pps)
            else:
                self.pps = init_patterns2(location)

    def is_in(self, line):
        '''
        检查输入中是否命中规则
        :param line: str or unicode
        :return: 命中返回True
        '''
        if isinstance(line, str):
            line = unistr(line)
        if not isinstance(line, string_types):
            raise Exception("输入必须是str或unicode")
        for name, (ps1, ps2) in self.pps.items():
            t1 = regex(ps1, line)
            t2 = regex(ps2, line)
            if (len(ps1) == 0 or t1 != True) and t2 == True:  # 不满足1的同时满足2才行
                # print line.encode('utf-8'),name.encode('utf-8')
                return True
        return False

    def hit_num(self, line):
        '''
        计算输入命中规则的次数
        :param line: str or unicode
        :return: int, 命中次数
        '''
        if isinstance(line, str):
            line = unistr(line)
        if not isinstance(line, string_types):
            raise Exception("输入必须是str或unicode")
        t = 0
        for name, (ps1, ps2) in self.pps.items():
            t1 = regex(ps1, line)
            t2 = regex(ps2, line)
            if (len(ps1) == 0 or t1 != True) and t2 == True:  # 不满足1的同时满足2才行
                t += 1
        return t

    def hit_group(self, line):
        hit_group = []
        for name, (ps1, ps2) in self.pps.items():
            t1 = regex(ps1, line)
            t2 = regex(ps2, line)
            if (len(ps1) == 0 or t1 != True) and t2 == True:  # 不满足1的同时满足2才行
                hit_group.append(name)
        return hit_group

        # def hit_num(self, line):
        #     return len(self.hit_group(line))


if __name__ == '__main__':
    p = PatternHiter("../../example/process_pattern_regex.exp")
    print(p.hit_num("世界和平"))
    print(p.is_in(u"世界和平"))



# coding: utf-8

# 此模块用以进行流处理

import sys
import codecs
import os


def checkDir(path, mode=0o777):
    dirname = os.path.dirname(path)
    if not dirname or os.path.exists(dirname):
        return
    os.makedirs(dirname, mode)


def rf_open(filename, encoding='utf-8', **kwargs):
    if sys.version_info[0] == 3:
        return open(filename, encoding=encoding, **kwargs)
    else:
        return open(filename, **kwargs)


def wf_open(stream_name='', mode="w", encoding="utf-8"):
    '''
    打开一个codecs流
    :param stream_name: str，默认为空
    :param mode: str，默认为 写 模式
    :param encoding: str，编码方式，默认为 utf-8
    :return: 返回打开的流，stream_name为空时 mode - w 返回标准错误输出; 否则，返回标准输出，不为空时，返回文件流
    '''
    if not stream_name:
        if mode == "w":
            return sys.stderr
        else:
            return sys.stdout
    elif isinstance(stream_name, (str, unicode, bytes)):
        checkDir(stream_name)
        return codecs.open(stream_name, mode=mode, encoding=encoding)
    else:
        raise TypeError("wf_open: %s" % stream_name)


def wf_close(stream):
    if stream in {sys.stdin, sys.stdout, sys.stderr}:
        return True
    else:
        try:
            stream.close()
        except:
            raise Exception('wf_close: %s' % stream)



# coding: utf-8

import time


class Clocker(object):
    def __init__(self):
        self.st = 0
        self.et = 0
        self.unit = 's'

    def start(self):
        self.st = time.clock()
        return self.st

    def end(self):
        self.et = time.clock()
        return self.et - self.st

    def interval(self):
        return self.et - self.st


if __name__ == '__main__':
    clocker = Clocker()
    clocker.start()
    time.sleep(3)
    print(clocker.end())



# coding: utf-8
import logging
import time
import os

MIDNIGHT = 24 * 60 * 60  # 00:00:00
SECONDS_PER_DAY = 60 * 60 * 24

'''
日志设定文件
'''


def keyboard_change_level(logger):
    import signal
    def warn_logging_level(signum, frame):
        logger.setLevel(logging.INFO)

    def info_logging_level(signum, frame):
        logger.setLevel(logging.WARN)

    signal.signal(signal.SIGUSR1, warn_logging_level)
    signal.signal(signal.SIGUSR2, info_logging_level)


def logger_params_set(logger, params):
    if "level" in params:
        logger.setLevel(params['level'])


def file_and_screen_logger(filename, file_params={}, console_params={}, logger_name=""):
    logger = logging.getLogger(logger_name)
    console = logging.StreamHandler()
    logger_params_set(console, console_params)
    filer = logging.FileHandler(filename)
    logger_params_set(filer, file_params)
    logger.addHandler(console)
    logger.addHandler(filer)
    return logger


def simple_stream_logger(logger_name, logger_level=logging.WARN, format='%(asctime)s, %(levelname)s %(message)s'):
    logger = logging.getLogger(logger_name)
    sh = logging.StreamHandler()
    if format:
        sh.setFormatter(logging.Formatter(format))
    logger.addHandler(sh)
    logger.setLevel(logger_level)
    logger.propagate = False
    return logger


# '%(asctime)s, %(levelname)s %(message)s'
def config_logging(filename=None, format='%(name)s, %(levelname)s %(message)s', level=logging.INFO,
                   logger=None, console_log_level=None, propagate=True, multi_process_logger_kwargs={},
                   file_format=None):
    if logger is None:
        logger = logging.getLogger()
    elif isinstance(logger, str):
        logger = logging.getLogger(logger)
    # need a clean state, for some module may have called logging functions already (i.e. logging.info)
    # in that case, a default handler would been appended, causing undesired output to stderr
    for handler in logger.handlers:
        logger.removeHandler(handler)
    formatter = logging.Formatter(format)
    logger.setLevel(level)
    if not propagate:
        logger.propagate = False
    if filename:
        if 'when' not in multi_process_logger_kwargs:
            multi_process_logger_kwargs['when'] = 'midnight'
        handler = MultiProcessRotatingFileHandler(filename=filename, **multi_process_logger_kwargs)
        file_formatter = formatter
        if file_format:
            file_formatter = logging.Formatter(file_format)
        handler.setFormatter(file_formatter)
        logger.addHandler(handler)
    if console_log_level is not None:
        ch = logging.StreamHandler()
        ch.setFormatter(logging.Formatter(format))
        ch.setLevel(console_log_level)
        logger.addHandler(ch)

    return logger


from logging.handlers import TimedRotatingFileHandler


class MultiProcessRotatingFileHandler(TimedRotatingFileHandler):
    def __init__(self, filename, when='h', interval=1, backupCount=0, encoding=None, utc=False):
        super(MultiProcessRotatingFileHandler, self).__init__(filename, when, interval, backupCount, encoding, False,
                                                              utc)
        d, f = os.path.split(filename)
        self.lock_file_name = os.path.join(d, '.' + f)

    def computeRollover(self, currentTime):
        """
        Work out the rollover time based on the specified time.
        都在整数时间点rollover
        """
        if self.when == 'MIDNIGHT' or self.when.startswith('W'):
            if self.utc:
                t = time.gmtime(currentTime)
            else:
                t = time.localtime(currentTime)
            currentHour = t[3]
            currentMinute = t[4]
            currentSecond = t[5]
            secondsToMidnight = MIDNIGHT - ((currentHour * 60 + currentMinute) * 60 +
                                            currentSecond)
            result = currentTime + secondsToMidnight
            if self.when.startswith('W'):
                day = t[6]
                if day != self.dayOfWeek:
                    if day < self.dayOfWeek:
                        daysToWait = self.dayOfWeek - day
                    else:
                        daysToWait = self.dayOfWeek - day + 7
                    result += SECONDS_PER_DAY * daysToWait
        else:
            result = currentTime + self.interval - currentTime % self.interval
        return result

    def doRollover(self):
        from filelock import FileLock
        """
        do a rollover; in this case, a date/time stamp is appended to the filename
        when the rollover happens.  However, you want the file to be named for the
        start of the interval, not the current time.  If there is a backup count,
        then we have to get a list of matching filenames, sort them and remove
        the one with the oldest suffix.
        rollover时两种情况:
        1、检查要被rename to的文件是否已经存在，如果已经存在，说明有另外的进程已经rollover了，那本进程
            a）reopen
            b) 更新rollover时间
        2、如果不存在，说明本进程是最先抢到rollover锁的，那本进程：
            a) rename
            b) 删除旧日志
            c) reopen
            d) 更新rollover时间
        """
        if self.stream:
            self.stream.close()
            self.stream = None
        # get the time that this sequence started at and make it a TimeTuple
        currentTime = int(time.time())
        t = self.rolloverAt - self.interval
        if self.utc:
            timeTuple = time.gmtime(t)
        else:
            timeTuple = time.localtime(t)
        dfn = self.baseFilename + "." + time.strftime(self.suffix, timeTuple)

        with FileLock(self.lock_file_name):
            if not os.path.exists(dfn):
                self._rollover(dfn)
        self.stream = self._open()
        self._updateRolloverAt(currentTime)

    def _rollover(self, dfn):
        # Issue 18940: A file may not have been created if delay is True.
        if os.path.exists(self.baseFilename):
            os.rename(self.baseFilename, dfn)

        if self.backupCount > 0:
            for s in self.getFilesToDelete():
                os.remove(s)

    def _updateRolloverAt(self, currentTime):
        """
        更新下一次rollover的时间点
        :param currentTime:
        :return:
        """
        newRolloverAt = self.computeRollover(currentTime)
        while newRolloverAt <= currentTime:
            newRolloverAt = newRolloverAt + self.interval
        self.rolloverAt = newRolloverAt


if __name__ == '__main__':
    file_and_screen_logger("temp/utilog.log")
    logging.info("test info")
    logging.warn("test warning")
    logging.error("test error")



# coding: utf-8
# created by tongshiwei on 17-12-17
from distutils.core import setup
from setuptools import find_packages

setup(
    name='longling',
    version='0.0.1',
    author='Sherlock, Shiwei Tong',
    author_email='tongsw@mail.ustc.edu.cn',
    packages=find_packages(),
    scripts=[],
    url='https://gitlab.com/tswsxk/longling.git',
    license='LICENSE.txt',
    description='handy wrapper for many libs',
    long_description=open('README.txt').read(),
    install_requires=[
    ],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],

)



# coding: utf-8
# created by tongshiwei on 17-12-17

from __future__ import absolute_import

from .lib import clocker
from .lib import process_pattern
from .lib import stream
from .lib import utilog

from .base import *
from .lib.stream import *
